/**
 *
 * \file client.c	
 *  Implements basic configuration and recieve/send functionality with Sensinode nRouted using
 *  the nrp protocol.
 *  Author: Matej Kristan (2009)
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h> 
#include "libnRP/libnRP.h"
 

// structure definition for the nrp protocol
struct nPing_opts_t
{
	char nRd_address[32];
	unsigned int nRd_port;
	
	unsigned char target[8];

	unsigned char verbose;
	unsigned char flood;
	int intereturn_val;
	int count;
	unsigned short int payload;
};
 

/** Main function takes two arguments: IP and port number where nRouted in listening*/
int main(int argc, char *argv[])
{
  unsigned short int tmp_port ; 
  int mymesslength ; 
  int sockfd, portno ;
  int k ;

  unsigned int packet_length, data_length;
  struct sockaddr_in serv_addr;
  struct hostent *server;  
  struct nPing_opts_t nPing_opts = 
  { "127.0.0.1", 21870, {0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff}, 0, 0, -1, -1, 2}; 
 
  unsigned char snd_rd_buf [128] ; // buffer to store outgoing or incoming packet
  unsigned char databuf[128] ; // buffer to store the data 
  int return_val ;
  unsigned short int source_port = 253;
  int tmp_val, tmp_val2 ;
 
// if there are no input argument, printout info and exit otherwise start the client
  if( argc < 2 ) {
    printf("---------------------------------------------------------------------------------------\n") ;
    printf("Application client.\n") ;
    printf("Connects to nRouted (must be running) on <IP> and <port number>\n") ;
    printf("Implements configuration of nRouted and basic send/recieve.\n") ;
    printf("After a successful configuration, the client constructs and sends a packet to all nodes.\n") ;
    printf("It then waits for reply.\n") ; printf("\n") ;
    printf("Proper usage:\n") ;
    printf("    ./client <IP> <port number>\n") ;
    printf("     <IP>          ... IP address of the machine which runs the nRouted.\n") ;
    printf("     <port number> ... port number on which the nRouted is listening.\n") ;
    printf("\n") ;
    printf("Teaching material for 'Sistemi Daljinskega Vodenja', Author: Matej Kristan (2011).\n") ;
    printf("---------------------------------------------------------------------------------------\n") ;
    printf("\n") ;
    return(0) ;
  }
  printf("\n") ; printf("----------------------------.\nClient starting.\n\n") ;

// initialize a socket for communication with nRouted
  portno = atoi(argv[2]);
  sockfd = socket(AF_INET, SOCK_STREAM, 0);
          
  server = gethostbyname(argv[1]);
          
  bzero((char *) &serv_addr, sizeof(serv_addr));
  serv_addr.sin_family = AF_INET;
  bcopy((char *)server->h_addr, 
         (char *)&serv_addr.sin_addr.s_addr,
         server->h_length); 
  serv_addr.sin_port = htons(portno);
  printf("Connecting to server... \n") ;
// socket initialized 

// connect the socket to address
  if (connect(sockfd,  (struct sockaddr *) &serv_addr,sizeof(serv_addr)) < 0) {
      printf("ERROR connecting to address!\n"); fflush(stdout) ;
      close(sockfd);
      return(-1);
  }
// connect the socket to address

// create a configuration nrp paket to forward packets to here
  tmp_port = 61630 ; 
  packet_length = libnrp_create_conf_pkt(snd_rd_buf, PROTO_6LOWPAN, NULL, NULL, ADDR_UNDEFINED, NULL, tmp_port) ;
 
// print out configuration packet 
  printf("--> Configuration message (first %d chars): ",packet_length);
  for( k = 0 ; k < packet_length; k++) {
      printf("0%x ",snd_rd_buf[k]);	
  } printf("\n");

// send the configuration packet to nRouted
  write(sockfd, snd_rd_buf, packet_length);

// wait for the configuration reply
  packet_length = read(sockfd, snd_rd_buf, 128);

// print out configuration reply packet
  printf("--> Received message (first %d chars): ",packet_length);
  for( k = 0 ; k < packet_length; k++) {
    printf("0%x ",snd_rd_buf[k]);	fflush(stdout) ;
  } ; printf("\n"); 
 
// verify if there has been an error in nRouted configuration process 
  if((return_val = libnrp_check_nRoute_reply(snd_rd_buf, packet_length)) != 1){
    printf("nRouted configuration error.\n"); fflush(stdout) ;
    close(sockfd);
    return(-1);
  }
 
/* nRouted is now correctly configured and the ready to send/recieve data */
// let's send something to the node now, and wait for node to send something back //

  // clear data buffer, write some packet and send the packet to the node
  bzero(databuf, 128);
  databuf[0] = 'H' ; databuf[1] = 'e' ; databuf[2] = 'l' ; databuf[3] = 'l' ;
  databuf[4] = 'o' ; databuf[5] = '!' ;
  data_length = 6 ;
 
// create a nrp packet which contains my data
  tmp_port = 61630 ;  
  packet_length = libnrp_create_data_pkt_hdr(snd_rd_buf, databuf, data_length, 
					PROTO_6LOWPAN, NULL, nPing_opts.target, 
					ADDR_IEEE_802_15_4_DEV_LONG, 
					&source_port, tmp_port, NULL) ;
  printf("--> Packet created with content: ");
// print out the nrp packet content
  for( k = 0 ; k < packet_length; k++) {
       printf("0x%x ",snd_rd_buf[k]);	
  } printf("\n") ; 
  fflush(stdout) ;

// send the packet to nRouted
  printf("--> Sending the packet...\n") ; 
  write(sockfd, snd_rd_buf, packet_length);
 
// wait for a reply packet
  printf("--> Waiting for packet... \n") ; fflush(stdout) ;
  packet_length = read(sockfd, snd_rd_buf, 512); // read at most 512 bytes
   
// printout the raw nrp recieved packet    
  printf("--> Recieved packet with content: ") ;
  for( k = 0 ; k < packet_length ; k++) {
	printf("0x%x ",snd_rd_buf[k]);	
   } ; printf("\n") ;
 
// parse the nrp packet and extract the data contained
  return_val = libnrp_get_data(snd_rd_buf, packet_length, databuf, &data_length );
  printf("--> Data contains %d bytes\n", data_length ) ;
  printf("--> Data content in hex: " ) ;
  for( k = 0 ; k < data_length ; k++) {
	printf("0x%x ",databuf[k]);	
  } ; printf("\n") ;
  printf("--> Data content in char: >>" ) ;
  for( k = 0 ; k < data_length ; k++) {
	printf("%c",databuf[k]);	
  } ; printf("<<\n") ;
 

// close the socket and exit
 close(sockfd);
 return 0;
}
