/**
 *
 * \file main.c
 *  Derived from the Sensionde's "nano_skeleton.c" 	
 *  Author: Matej Kristan (2011)
 */

/* Standard includes. */
#include <stdlib.h>
#include <string.h>
#include <sys/inttypes.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"

/* NanoStack includes */
#include "stack.h"
#include "socket.h"
#include "debug.h"
#include "control_message.h"

/* Platform includes */
#include "uart.h"
#include "rf.h"
#include "bus.h"
#include "dma.h"
#include "timer.h"
#include "gpio.h"

#include "readsensors/readsensors.h"
extern uint16_t val_temperatura ;
extern uint16_t val_svetlost ;

// global variables
stack_event_t stack_dogodki ;
buffer_t *moj_buffer ;
uint8_t skeniraj ;
portTickType t0 ;
portTickType t1 ;
 
sockaddr_t naslov_gw = 
{
  ADDR_802_15_4_PAN_LONG,
  {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
  61630
} ;

address_t moj_router = {0x00, 0x15, 0x20, 0x00, 0x00, 0x02, 0x10, 0xFF, 0xFF,0xFF} ;

socket_t *com_socket ;
stack_event_t stack_event;


// my functions
static void mojaGlavnaFunkcija( int8_t *pvParameters );
 
/* Main task, initialize hardware and start the FreeRTOS scheduler */
int main( void )
{
	/* Initialize the Nano hardware */
	  // initialize leds
	  LED_INIT();
	  // initialize hardware
	  bus_init() ;
	  
	  // initialize stack variables
	  stack_init() ;
	/* Setup the application task and start the scheduler */
	xTaskCreate( mojaGlavnaFunkcija, "App", configMAXIMUM_STACK_SIZE, NULL, 
	             (tskIDLE_PRIORITY + 2 ), ( xTaskHandle * )NULL ) ;
	vTaskStartScheduler();
	
	/* Scheduler has taken control, next vAppTask starts executing. */
	return 0;
}

/** 
 * Skeleton application task 
 */
static void mojaGlavnaFunkcija( int8_t *pvParameters )
{		
	int16_t znak ;
	int8_t current_channel ;
	portTickType ticks_from_start ;
	uint16_t tmp_uint16 ;
	uint8_t *pData ;

	pvParameters ; // just so that the compiler doesn't complain.

	// initialize debug stuff
	debug_init(115200) ;
	vTaskDelay( 200 / portTICK_RATE_MS );

	/* Socket initialization code */
	// Open socket for stac status messages
	stack_event = stack_service_init(NULL) ;
	// connect socket
	com_socket = socket(MODULE_CUDP, 0) ;
	// start listening to socket
	socket_bind(com_socket, &naslov_gw) ; 
       /* Socket initilized */


	/* Start the debug UART at 115k */
	LED1_ON() ; LED2_ON() ;
	vTaskDelay( 500 / portTICK_RATE_MS ) ;
	LED1_OFF() ; LED2_OFF() ;

	t0 = xTaskGetTickCount() ;
	skeniraj = 1 ;
	stack_dogodki = stack_service_init(NULL) ;
	// enter infinite loop 
	for (;;) {

	  /* read from socket for 300 cycles */
	  if ( skeniraj == 0 ) {
	    //debug("Skeniram vhod...\r\n") ;
	    moj_buffer = socket_read(com_socket, 300) ;
	    if ( moj_buffer ) {
	      // preberi buffer
	      tmp_uint16 = buffer_data_length( moj_buffer ) ;	   
	      pData = buffer_data_pointer( moj_buffer ) ; 

	      debug("\r\nNekaj sem dobil na socket!\r\n") ;	      
	      debug("Content: ") ; debug_send( pData, tmp_uint16 ) ; debug("\r\n") ;
  
	      // tukaj dodaj kodo za analizo vsebine *pData




	      // sprosti buffer
	      socket_buffer_free(moj_buffer) ;
	      moj_buffer = 0 ;
 
	      // sedaj lahko ponovno rezerviramo buffer za odgovor	      
	      while( moj_buffer == 0 ) {
			moj_buffer = socket_buffer_get( com_socket ) ;
	      }
	      // get pointer to buffer and clear
	      pData = buffer_data_pointer( moj_buffer ) ; 
 
	      // napolni pData z izhodnimi podatki -- primer:
	      pData[0] = 'H' ; pData[1] = 'i' ; pData[2] = '!' ;
	      moj_buffer->buf_end = 3 ;
 

	      naslov_gw.port = 61630;
	      // send to destination
	      debug("Posiljam paket... ") ;	 
	      //debug("na port ") ; debug_int(naslov_gw.port) ; debug(" \r\n") ;
	      if (socket_sendto(com_socket, &naslov_gw, moj_buffer) != pdTRUE) {			  
			debug("ni uspelo!\r\n") ;
			// manually free buffer
			socket_buffer_free(moj_buffer); 
			moj_buffer = 0 ;  
	      } else {
			debug("uspelo!\r\n") ;
			// the buffer is automatically freed upon success!
	      }
	    }
	  }

	  /* here comes the main code */
	  znak = debug_read_blocking( 100/portTICK_RATE_MS ) ;
	  if( znak != -1 ) {
	  switch(znak) {
	      case 'w':
		current_channel = mac_current_channel() ;
		debug("Trenutna vrednost kanala je: ") ;
		debug_int( current_channel ) ;
		debug("\r\n") ;  
		break ;
	      case 'c':
		current_channel = mac_current_channel() ;
		current_channel-- ;		
		if( current_channel < 11 ) {
		  current_channel = 11 ;    
		}     
		mac_set_channel( current_channel ) ;
		debug("Trenutna vrednost kanala je: ") ;
		debug_int( current_channel ) ;
		debug("\r\n") ;
		break ;
	      case 'C':
		current_channel = mac_current_channel() ;
		current_channel++ ;
		if( current_channel > 26 ) {
		  current_channel = 26 ;    
		}
		mac_set_channel( current_channel ) ;
		debug("Trenutna vrednost kanala je: ") ;
		debug_int( current_channel ) ;
		debug("\r\n") ;
		break ;
	      case 't':
		ticks_from_start = xTaskGetTickCount() ;
		debug("Preteklo stevilo sekund od zacetka: ") ;
		debug_int( ticks_from_start*portTICK_RATE_MS/1000 ) ;
		debug("\r\n") ;
		break ;
	      case 's':
		skeniraj = 1 ;	 
		break ;
	      case 'g':
		debug("MAC naslov shranjenega gatewaya:") ;
		debug_address(&naslov_gw) ;
		debug("\r\n"); 
	    } 
	  }


 
	  moj_buffer = waiting_stack_event(300) ; 
	  if( moj_buffer != 0 ) {
	    switch (parse_event_message(moj_buffer)) {
	      case BROKEN_LINK:
		debug("Izgubil sem gateway!") ; debug("\r\n");
		naslov_gw.addr_type = ADDR_NONE;
		skeniraj = 1 ; 
		break ;
	      case ROUTER_DISCOVER_RESPONSE:
		if ( skeniraj == 1 ) {
		  current_channel = mac_current_channel() ;
		  debug("Nasel sem gateway na kanalu:") ;
		  debug_int( current_channel ) ; debug("\r\n"); 
		  debug("MAC routerja: ");
		  //debug_send(&(moj_buffer->src_sa.address), sizeof(address_t));
		  debug_address(&naslov_gw) ;
		  debug("\r\n");
		  debug_address(&(moj_buffer->src_sa)) ;
		  debug("\r\n"); 
		
		  if ( memcmp(moj_router, &(moj_buffer->src_sa.address), sizeof(address_t)) == 0 ) {
		    debug("Nasel sem nas router!\r\n") ;
		    skeniraj = 0 ;  
		    memcpy(&naslov_gw, &(moj_buffer->src_sa), sizeof(sockaddr_t));
		   // memcpy(naslov_gw.address, moj_buffer->src_sa.address, sizeof(address_t)) ;   
		    //naslov_gw.addr_type = moj_buffer->src_sa.addr_type ;
		    //naslov_gw.port = moj_buffer->src_sa.port ;
		  }
		}
		break ;	    
	    }
	    stack_buffer_free( moj_buffer ) ;
	  } else {
	    moj_buffer = 0 ;
	  }

	  if ( skeniraj == 1 ) {
	    t1 = xTaskGetTickCount() ;
	    if ( (t1 - t0)*portTICK_RATE_MS > 1000 ) {
	      current_channel = mac_current_channel() ;
	      current_channel++ ;
	      if ( current_channel > 26 ) {
		current_channel = 11 ;
	      }
	      debug("Preklapljam na kanal: ") ; debug_int(current_channel) ; debug("\r\n") ;
	      mac_set_channel( current_channel ) ;
	      gw_discover() ;
	      t0 = t1 ;
	    }
	  } 


	  /* end of the main code */	
	  /* Sleep for 500 ms */
	  if ( skeniraj == 1 ) {
	    vTaskDelay( 500 / portTICK_RATE_MS );
	    LED2_TOGGLE() ; // wink the led2 
	  } else {
	    LED2_ON() ;  
	  }
	}     	
}
