/** 
 * Functions for reading the sensory data 
 */ 
#include "readsensors.h" 
 
uint16_t val_temperatura = 0 ; // sensor1 16bit value
uint16_t val_svetlost = 0 ; // sensor2 16bit value

  
int8_t readSensors()
{
	/*
		Reads raw sensor values and transforms them to Kelvins and Lux .
		Returns 0 on success and -1 on fail.
	*/
	uint32_t sum = 0 ; // temporary 32bit value for sensors
	int8_t retval ;
 	taskENTER_CRITICAL() ;
 	val_temperatura = 0 ; // clear sensor values
 	val_svetlost = 0 ;
 
 	// read temperature sensor voltage
 	retval = get_adc_value(N710_TEMP, &val_temperatura) ;
 	// transform the voltage to temperature
 	sum = (int32_t)val_temperatura ;
 	sum /= 4 ;      // drop insignificant bits
 	sum *= 24420 ;    // 1.25*160.039*1000*1000/8192 (value with         6 decimals)
 	sum /= 10000 ;   // adjust to 2 decimals 
 	sum -= 6785 ; 
 	val_temperatura = (uint16_t) sum ;
 	val_temperatura /= 100 ;
 	//val_temperatura += 273 ;

 	// read light sensor			
 	retval += get_adc_value(N710_LIGHT, &val_svetlost) ;
 	sum =((val_svetlost * 5) / 99) ;
 	val_svetlost = (uint16_t) sum ; 
 	taskEXIT_CRITICAL() ;	
 	
 	if ( retval < 0 ){ retval = -1 ; }
 	return retval ;
}  

void clearSensors() {
	val_temperatura = 0 ;
	val_svetlost = 0 ;	
}

int8_t get_adc_value(adc_input_t channel, uint16_t *value)
{
 /*
 	Returns 0 on success and -1 on fail
 */	
	int8_t retval ;
	if (adc_convert_single(channel, ADCREF_125V, ADCRES_14BIT) == 0) 
	{
		while (retval != 1) 
		{
			pause_us(60);
			retval = 0;
			retval = adc_result_single(value);
		}
		retval = 0;
	}
	else
	{
		retval = -1;
	}
	
	return retval;
}
